﻿// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//                                       Digit.as
//                                       ________
//
//                   Created by William Malone (www.williammalone.com)                
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

package com.williammalone
{	
	import flash.display.Sprite;
    import flash.display.Shape;

	public class Digit extends Sprite
	{
		//-------------------------------------------------------------------------------
		//                                                                         FIELDS
		//-------------------------------------------------------------------------------
		private static const A:int = 0;
		private static const B:int = 1;
		private static const C:int = 2;
		private static const D:int = 3;
		private static const E:int = 4;
		private static const F:int = 5;
		private static const G:int = 6;
		
		private static const NONE:int = -1;
		
		private var segmentColor:uint = 0x000000;
		private var digitHeight:Number = 80;
		private var digitWidth:Number;
		private var segmentRatio:Number = 2/5;
		private var thicknessRatio:Number = 3/20;
		private var paddingRatio:Number = 1/40;
		private var segmentThickness:Number;
		private var segmentPadding:Number;
		private var segmentPaddingCartesian:Number;
		private var segmentLength:Number;
		private var digitValue:int;
		private var segments:Array = new Array();
		private var glyphs:Array = new Array();	
		//-------------------------------------------------------------------------------
		//                                                                    CONSTRUCTOR
		//-------------------------------------------------------------------------------
		public function Digit(pHeight:Number = NONE, pParent:Sprite = null, pDigitValue:Number = NONE) 
		{
			if(pParent != null){
				pParent.addChild(this);
			}
			
			if(pHeight != NONE){
				digitHeight = pHeight;
			}
			
			if(pDigitValue >= 0 && pDigitValue < 10){
				digitValue = pDigitValue;
			}else{
				digitValue = NONE;
			}
			
			create();
		}
		//-------------------------------------------------------------------------------
		//                                                                        METHODS
		//-------------------------------------------------------------------------------
		/********************************************************************************
		*  create
		*/
		private function create():void
		{
			// Glyph array (0-9) indicating value of each segment to create that glyph
			glyphs[0] = new Array(true, true, true, true, true, true, false);
			glyphs[1] = new Array(false, true, true, false, false, false, false);
			glyphs[2] = new Array(true, true, false, true, true, false, true);
			glyphs[3] = new Array(true, true, true, true, false, false, true);
			glyphs[4] = new Array(false, true, true, false, false, true, true);
			glyphs[5] = new Array(true, false, true, true, false, true, true);
			glyphs[6] = new Array(true, false, true, true, true, true, true);
			glyphs[7] = new Array(true, true, true, false, false, false, false);
			glyphs[8] = new Array(true, true, true, true, true, true, true);
			glyphs[9] = new Array(true, true, true, true, false, true, true);								
			
			// For each segment (a-g)
			var i:int = A;
			for(; i <= G + 1; i++)
			{
				segments[i] = new Object();
			}
			
			// Define each segment's angle
			segments[A].angle = 0;
			segments[B].angle = 90;
			segments[C].angle = 90;
			segments[D].angle = 0;
			segments[E].angle = 90;
			segments[F].angle = 90;
			segments[G].angle = 0;
			
			// For each segment (a-g)
			for(i = A; i <= G; i++)
			{
				segments[i].container = new Shape();
				addChild(segments[i].container);
			}
			
			// Update the value for the first time
			updateValue();
			// Draw the digit segments for the first time
			redraw();
		}
		/********************************************************************************
		*  redraw
		*/
		private function redraw():void
		{
			// Update some variables that may have changed
			digitWidth = digitHeight / 2;
			segmentThickness = digitHeight * thicknessRatio;
			segmentPadding = digitHeight * paddingRatio;
			segmentLength = segmentThickness / segmentRatio;
			segmentPaddingCartesian = segmentPadding * Math.sqrt(2) / 2;
			
			// Define each segment's position
			segments[A].x = segmentThickness / 2 + segmentPaddingCartesian;
			segments[A].y = 0;

			segments[B].x = segments[A].x + segmentLength - segmentThickness / 2 + segmentPaddingCartesian;
			segments[B].y = segments[A].x;

			segments[C].x = segments[B].x;
			segments[C].y = segments[B].y + segmentLength + 2 * segmentPaddingCartesian;
			
			segments[D].x = segments[A].x;
			segments[D].y = segments[C].y + segmentLength - segmentThickness / 2 + segmentPaddingCartesian;

			segments[E].x = 0;
			segments[E].y = segments[C].y;

			segments[F].x = 0;
			segments[F].y = segments[B].y;
			
			segments[G].x = segments[A].x;
			segments[G].y = segments[B].y + segmentLength - segmentThickness / 2 + segmentPaddingCartesian;
			
			// For each segment (a-g)
			var i:int = A;
			for(; i <= G; i++)
			{
				// Position each segment
				segments[i].container.x = segments[i].x;
				segments[i].container.y = segments[i].y;

				// Draw the segment
				segments[i].container.graphics.clear();
				segments[i].container.graphics.beginFill(segmentColor);
				if(!segments[i].angle)
				{
					segments[i].container.graphics.moveTo(0, segmentThickness / 2);
					
					segments[i].container.graphics.lineTo(segmentThickness / 2, 0);
					segments[i].container.graphics.lineTo(segmentLength - segmentThickness / 2, 0);
					segments[i].container.graphics.lineTo(segmentLength, segmentThickness / 2);
					segments[i].container.graphics.lineTo(segmentLength - segmentThickness / 2, segmentThickness);
					segments[i].container.graphics.lineTo(segmentThickness / 2, segmentThickness);
					segments[i].container.graphics.lineTo(0, segmentThickness / 2);
				}
				else
				{
					segments[i].container.graphics.moveTo(segmentThickness / 2, 0);
					
					segments[i].container.graphics.lineTo(0, segmentThickness / 2);
					segments[i].container.graphics.lineTo(0, segmentLength - segmentThickness / 2);
					segments[i].container.graphics.lineTo(segmentThickness / 2, segmentLength);
					segments[i].container.graphics.lineTo(segmentThickness, segmentLength - segmentThickness / 2);
					segments[i].container.graphics.lineTo(segmentThickness, segmentThickness / 2);
					segments[i].container.graphics.lineTo(segmentThickness / 2, 0);
				}
				segments[i].container.graphics.endFill();
			}
		}
		/********************************************************************************
		*  updateValue
		*/
		private function updateValue():void
		{
			// For each segment (a-g)
			var i:int = A;
			for(; i <= G; i++)
			{
				// Change the alpha of each segment based on which glyph corresponds with the digit's value
				segments[i].container.alpha = (digitValue != NONE && glyphs[digitValue][i]) ? 0.9 : 0.2;
			}
		}
		/********************************************************************************
		*  addParent
		*/
		public function addParent(pParent:Sprite):void
		{		
			if(pParent != null)
			{
				pParent.addChild(this);
			}else{
				throw new Error("Parameter pParent is null.  Adoption has failed. (Digit.as :: adopt)");
			}
		}
		//-------------------------------------------------------------------------------
		//                                                                     PROPERTIES
		//-------------------------------------------------------------------------------		
		/********************************************************************************
		*  get color
		*/
		public function get color():uint
		{
			return segmentColor;
		}
		/********************************************************************************
		*  set color
		*/
		public function set color(pValue:uint):void
		{
			segmentColor = pValue;
			redraw();
		}
		/********************************************************************************
		*  get value
		*/
		public function get value():int
		{
			return digitValue;
		}
		/********************************************************************************
		*  set value
		*/
		public function set value(pValue:int):void
		{
			if(pValue >= 0 && pValue < 10){
				digitValue = pValue;
			}else{
				digitValue = NONE;
			}
			updateValue();
		}
		/********************************************************************************
		*  get color
		*/
		override public function get width():Number
		{
			return digitWidth;
		}
		/********************************************************************************
		*  set width
		*/
		override public function set width(pValue:Number):void
		{
			throw new Error("Digit width cannot be set.  Height can be set. (Digit.as :: set width)");
		}
		/********************************************************************************
		*  get height
		*/
		override public function get height():Number
		{
			return digitHeight;
		}
		/********************************************************************************
		*  set height
		*/
		override public function set height(pValue:Number):void
		{
			digitHeight = pValue;
			
			redraw();
		}
		/********************************************************************************
		*  get thickness
		*/
		public function get thickness():Number
		{
			return segmentThickness;
		}
		/********************************************************************************
		*  set thickness
		*
		public function set thickness(pValue:Number):void
		{
			segmentThickness = pValue;
			
			redraw();
		}
		/********************************************************************************
		*  get padding
		*/
		public function get padding():Number
		{
			return segmentPadding;
		}
		/********************************************************************************
		*  set padding
		*
		public function set padding(pValue:Number):void
		{
			segmentPadding = pValue;
			
			redraw();
		}
		/**/
	}
}